package fmon.draw

import java.io.InputStream

import scalafx.geometry.Rectangle2D
import scalafx.scene.image._

case class Box(val x: Int, val y: Int, val width: Int, val height: Int)

case class StillImg(val image: Image, val x: Double, val y: Double, val width: Double, val height: Double) extends Drawable {
  def draw(view: ImageView, t: Double = 0): Unit = {
    if (view.image() != image.delegate) {
      view.image = image
    }
    view.viewport = new Rectangle2D(x, y, width, height)
  }
  
  def croppedImage(): Image = {
    new WritableImage(image.pixelReader.get, x.toInt, y.toInt, width.toInt, height.toInt)
  }
}

class Palette(val image: Image, val numAcross: Int, val numDown: Int) {
  def this(url: String, numAcross: Int, numDown: Int) = this(new Image(url), numAcross, numDown)
  def this(stream: InputStream, numAcross: Int, numDown: Int) = this(new Image(stream), numAcross, numDown)
  
  val imgWidth = image.width() / numAcross
  val imgHeight = image.height() / numDown
  
  def apply(x: Int, y: Int): StillImg = {
    StillImg(image, x * imgWidth, y * imgHeight, imgWidth, imgHeight)
  }
  
  def apply(x : Int, y : Int, w : Int, h : Int): StillImg = {
    StillImg(image, x * imgWidth, y * imgHeight, w * imgWidth, h * imgHeight)
  }
  
  /**
   * Creates a sub-image that spans multiple cells
   */
  def apply(cells : Box): StillImg = {
    this(cells.x, cells.y, cells.width, cells.height)
  }
}